@@ -1,9 +1,13 @@
+import copy
 import logging
 import subprocess as sp
 
 from collections import OrderedDict
+from pathlib import Path
+from typing import Dict, Sequence
 
 from Bio import SeqIO
+from Bio.Seq import Seq
 from Bio.SeqUtils.ProtParam import ProteinAnalysis
 
 import bakta.config as cfg
@@ -17,7 +21,7 @@ import bakta.so as so
 log = logging.getLogger('CDS')
 
 
-def predict(genome, sequences_path):
+def predict(genome: dict, sequences_path: Path):
     """Predict open reading frames with Prodigal."""
     # create prodigal trainining file if not provided by the user
     prodigal_tf_path = cfg.prodigal_tf
@@ -66,7 +70,7 @@ def predict(genome, sequences_path):
     return cdss
 
 
-def execute_prodigal(genome, contigs_path, traininng_file_path, proteins_path=None, gff_path=None, train=False, complete=False):
+def execute_prodigal(genome: dict, contigs_path: Path, traininng_file_path: Path, proteins_path: Path=None, gff_path: Path=None, train: bool=False, complete: bool=False):
     log.debug('execute-prodigal: contigs-path=%s, traininng-file-path=%s, proteins-path=%s, gff-path=%s, train=%s, complete=%s', contigs_path, traininng_file_path, proteins_path, gff_path, train, complete)
     cmd = [
         'prodigal',
@@ -109,7 +113,7 @@ def execute_prodigal(genome, contigs_path, traininng_file_path, proteins_path=No
         raise Exception(f'prodigal error! error code: {proc.returncode}')
 
 
-def parse_prodigal_output(genome, sequences, gff_path, proteins_path):
+def parse_prodigal_output(genome: dict, sequences, gff_path: Path, proteins_path: Path):
     log.debug('parse-prodigal-output: gff-path=%s, proteins-path=%s', gff_path, proteins_path)
     cdss = {}
     partial_cdss_per_record = {}
@@ -226,7 +230,7 @@ def parse_prodigal_output(genome, sequences, gff_path, proteins_path):
     return cdss
 
 
-def split_gff_annotation(annotation_string):
+def split_gff_annotation(annotation_string: str) -> Dict[str, str]:
     annotations = {}
     for expr in annotation_string.split(';'):
         if(expr != ''):
@@ -238,7 +242,7 @@ def split_gff_annotation(annotation_string):
     return annotations
 
 
-def predict_pfam(cdss):
+def predict_pfam(cdss: Sequence[dict]) -> Sequence[dict]:
     """Detect Pfam-A entries"""
     fasta_path = cfg.tmp_path.joinpath('hypotheticals.faa')
     orf.write_internal_faa(cdss, fasta_path)
@@ -298,9 +302,9 @@ def predict_pfam(cdss):
                 if('pfams' not in cds):
                     cds['pfams'] = []
                 cds['pfams'].append(pfam)
-                if('dbxrefs' not in cds):
-                    cds['dbxrefs'] = []
-                cds['dbxrefs'].append(f"PFAM:{pfam['id']}")
+                if('db_xrefs' not in cds):
+                    cds['db_xrefs'] = []
+                cds['db_xrefs'].append(f"PFAM:{pfam['id']}")
                 pfam_hits.append(cds)
                 cds_pfams_hits[aa_identifier] = cds
                 log.info(
@@ -311,7 +315,7 @@ def predict_pfam(cdss):
     return cds_pfams_hits.values()
 
 
-def analyze_proteins(cdss):
+def analyze_proteins(cdss: Sequence[dict]):
     for cds in cdss:
         seq = ProteinAnalysis(cds['aa'])
         seq_stats = OrderedDict()
@@ -334,16 +338,106 @@ def analyze_proteins(cdss):
         cds['seq_stats'] = seq_stats
 
 
-def revise_special_cases(cdss):
+def revise_translational_exceptions(genome: dict, cdss: Sequence[dict]):
+    """
+    Revise translational exceptions as for istance selenocystein proteins.
+    """
+    no_revised = 0
+    if(bc.FEATURE_NC_RNA_REGION not in genome['features']):  # check if ncRNA regions have been detected, otherwise skip analysis and return
+        return no_revised
+
+    contigs = {c['id']: c for c in genome['contigs']}
+    # detect splitted orphan ORFs of selenocystein proteins that are subject to stop codon recoding.
+    cdss_per_contigs = {k['id']: [] for k in genome['contigs']}  # get CDS per contig
+    for cds in cdss:
+        cdss_per_contig = cdss_per_contigs[cds['contig']]
+        if('truncated' not in cds):  # exclude truncated CDS for now
+            cdss_per_contig.append(cds)
+    cds_pairs_per_contig = {k['id']: [] for k in genome['contigs']}  # extract inframe primate CDS neighbouring pairs
+    for id, cdss_per_contig in cdss_per_contigs.items():
+        cdss_per_contig = sorted(cdss_per_contig, key=lambda k: k['start'])
+        for i in range(1, len(cdss_per_contig)):
+            cds_a = cdss_per_contig[i-1]
+            cds_b = cdss_per_contig[i]
+            strand = cds_a['strand']
+            upstream_stop_codon = cds_a['nt'][-3:] if strand == bc.STRAND_FORWARD else cds_b['nt'][-3:]
+            if(
+                cds_a['strand'] == cds_b['strand'] and  # up- and downstream ORFs on the same strand
+                cds_a['frame'] == cds_b['frame'] and  # up- and downstream ORFs on the same frame
+                upstream_stop_codon == 'TGA' and  # tRNAScan-SE 2.0 only predicts tRNA-Sec with UCA anticodons, therefore we can only detect TGA stop codons
+                (cds_b['start'] - cds_a['stop']) < 100):  # up- and downstream ORFs in close proximity
+                cds_pairs = cds_pairs_per_contig[cds_a['contig']]
+                cds_pairs.append((cds_a, cds_b))
+
+    recoding_regions = [ncrna_region for ncrna_region in genome['features'][bc.FEATURE_NC_RNA_REGION] if ncrna_region['class'] == so.SO_CIS_REG_RECODING_STIMULATION_REGION]  #  Selenocysteine insertion sequences
+    for recoding_region in recoding_regions:
+        if('selenocysteine' in recoding_region.get('product', '').lower()):
+            cds_pairs = cds_pairs_per_contig[recoding_region['contig']]
+            for cds_a, cds_b in cds_pairs:  # find CDS pair around recoding region
+                strand = cds_a['strand']
+                if(
+                    strand == recoding_region['strand'] and  # everything is on the same strand
+                    cds_a['start'] < recoding_region['start'] and recoding_region['stop'] < cds_b['stop']):  # recoding region lies between up- and downstream ORFs
+                    log.debug(
+                        'selenocysteine recoding ncRNA/CDS pair detected: contig=%s, strand=%s, CDS-A=[%i...%i] (%s..%s), recoding-ie=[%i..%i], CDS-B=[%i...%i] (%s..%s)',
+                        recoding_region['contig'], recoding_region['strand'], cds_a['start'], cds_a['stop'], cds_a['nt'][:10], cds_a['nt'][-10:], recoding_region['start'], recoding_region['stop'], cds_b['start'], cds_b['stop'], cds_b['nt'][:10], cds_b['nt'][-10:]
+                    )
+                    seleno_cds = copy.deepcopy(cds_a)
+                    seleno_cds['stop'] = cds_b['stop']
+                    seleno_cds['rbs_motif'] = cds_a['rbs_motif'] if strand == bc.STRAND_FORWARD else cds_b['rbs_motif']
+                    contig = contigs[seleno_cds['contig']]
+                    nt = bu.extract_feature_sequence(seleno_cds, contig)
+                    seleno_cds['nt'] = nt
+                    aa = str(Seq(nt).translate(table=cfg.translation_table, stop_symbol='*', to_stop=False, cds=False))
+                    if(
+                        aa[0] == 'M' and  # starts with M
+                        aa[-1] == '*' and  # ends with stop *
+                        aa[1:-1].count('*') == 1  # contains exactly 1 additional stop (*) somewhere in between
+                        ):
+                        aa = aa.replace('*', 'U', 1)  # replace internal stop codon by U -> selenocysteine
+                        aa = aa[:-1]  # remove stop asterisk
+                        seleno_cds['aa'] = aa
+                        seleno_cds['aa_digest'], seleno_cds['aa_hexdigest'] = bu.calc_aa_hash(aa)
+                        seleno_cds['exception'] = {
+                            'type': 'selenocysteine',
+                            'aa': 'Sec',
+                            'start': cds_a['stop'] - 2 if strand == bc.STRAND_FORWARD else cds_b['start'],
+                            'stop': cds_a['stop'] if strand == bc.STRAND_FORWARD else cds_b['start'] + 2,
+                            'codon_position': aa.find('U') + 1
+                        }                    
+                        cdss.append(seleno_cds)
+                        log.info(
+                            'selenocysteine CDS detected: contig=%s, start=%i, stop=%i, strand=%s, frame=%i, exception=[%i..%i], nt=[%s..%s], aa=[%s..%s], aa-hexdigest=%s',
+                            seleno_cds['contig'], seleno_cds['start'], seleno_cds['stop'], seleno_cds['strand'], seleno_cds['frame'], seleno_cds['exception']['start'], seleno_cds['exception']['stop'], nt[:10], nt[-10:], aa[:10], aa[-10:], seleno_cds['aa_hexdigest']
+                        )
+                        discard = {  # mark CDS a/b as discarded
+                            'type': bc.DISCARD_TYPE_RECODING,
+                            'description': f"selenocysteine recoding at ({seleno_cds['exception']['start']}..{seleno_cds['exception']['stop']})"
+                        }
+                        cds_a['discarded'] = discard
+                        cds_b['discarded'] = discard
+                        no_revised += 1
+                    else:
+                        log.warn(
+                            'spurious selenocysteine CDS detected: contig=%s, start=%i, stop=%i, strand=%s, frame=%i, nt=[%s], aa=[%s]',
+                            seleno_cds['contig'], seleno_cds['start'], seleno_cds['stop'], seleno_cds['strand'], seleno_cds['frame'], nt, aa
+                        )
+    return no_revised
+
+
+def revise_special_cases_annotated(genome: dict, cdss: Sequence[dict]):
     """
     Revise rare but known special cases as for istance supposedly truncated dnaA genes on rotated chromosome starts
     which often appear on re-annotated genomes.
     """
     
+    contigs = {c['id']: c for c in genome['contigs']}
     # look for supposedly truncated dnaA genes on rotated chromosome starts: start=1, strand=+
     dnaA = None
     for cds in cdss:
+        contig = contigs[cds['contig']]
         if(
+            contig['complete'] and
             cds['start'] == 1 and 
             cds['strand'] == bc.STRAND_FORWARD and 
             cds['start_type'] == 'Edge' and 
@@ -351,32 +445,32 @@ def revise_special_cases(cdss):
             ('dnaa' in cds['product'].lower().split() or cds['gene'] == 'dnaA')):
             dnaA = cds
             break
-    if(dnaA is not None):
+    if(dnaA is not None and 'truncated' in dnaA):
         dnaA.pop('truncated')
-        gene = dnaA.get('gene', None)
-        gene = gene if gene is not None else '-'
+        gene = dnaA.get('gene', '-')
         log.info(
             'revise supposedly truncated dnaA gene on rotated chromosome start: contig=%s, start=%i, stop=%i, strand=%s, gene=%s, product=%s, nt=[%s..%s], aa=[%s..%s]',
             dnaA['contig'], dnaA['start'], dnaA['stop'], dnaA['strand'], gene, dnaA['product'], dnaA['nt'][:10], dnaA['nt'][-10:], dnaA['aa'][:10], dnaA['aa'][-10:]
         )
     
     # look for supposedly truncated repA genes on rotated plasmid starts: start=1, strand=+
-    repA = None
+    repAs = []
     for cds in cdss:
+        contig = contigs[cds['contig']]
         if(
+            contig['complete'] and
             cds['start'] == 1 and 
             cds['strand'] == bc.STRAND_FORWARD and 
             cds['start_type'] == 'Edge' and 
             cds['rbs_motif'] is None and
             ('repa' in cds['product'].lower().split() or cds['gene'] == 'repA')):
-            repA = cds
-            break
-    if(repA is not None):
-        repA.pop('truncated')
-        gene = repA.get('gene', None)
-        gene = gene if gene is not None else '-'
-        log.info(
-            'revise supposedly truncated repA gene on rotated plasmid start: contig=%s, start=%i, stop=%i, strand=%s, gene=%s, product=%s, nt=[%s..%s], aa=[%s..%s]',
-            repA['contig'], repA['start'], repA['stop'], repA['strand'], gene, repA['product'], repA['nt'][:10], repA['nt'][-10:], repA['aa'][:10], repA['aa'][-10:]
-        )
+            repAs.append(cds)
+    for repA in repAs:
+        if('truncated' in repA):
+            repA.pop('truncated')
+            gene = repA.get('gene', '-')
+            log.info(
+                'revise supposedly truncated repA gene on rotated plasmid start: contig=%s, start=%i, stop=%i, strand=%s, gene=%s, product=%s, nt=[%s..%s], aa=[%s..%s]',
+                repA['contig'], repA['start'], repA['stop'], repA['strand'], gene, repA['product'], repA['nt'][:10], repA['nt'][-10:], repA['aa'][:10], repA['aa'][-10:]
+            )
     
\ No newline at end of file
