@@ -2,16 +2,23 @@ import logging
 import subprocess as sp
 
 from collections import OrderedDict
+from pathlib import Path
+from typing import Sequence
 
 import bakta.config as cfg
 import bakta.constants as bc
 import bakta.utils as bu
 
 
+HIT_COVERAGE = 0.8
+HIT_IDENTITY = 0.8
+HIT_EVALUE = '1E-5'
+
+
 log = logging.getLogger('ORI')
 
 
-def predict_oris(genome, contigs_path, ori_type):
+def predict_oris(genome: dict, contigs_path: Path, ori_type: str) -> Sequence[dict]:
     """Search for oriT/C sequences."""
 
     database = 'oric.fna' if ori_type == bc.FEATURE_ORIC else 'orit.fna'
@@ -21,7 +28,7 @@ def predict_oris(genome, contigs_path, ori_type):
         '-query', str(cfg.db_path.joinpath(database)),
         '-subject', str(contigs_path),
         '-culling_limit', '1',
-        '-evalue', '1E-5',
+        '-evalue', HIT_EVALUE,
         '-num_threads', str(cfg.threads),
         '-outfmt', '6 qseqid qstart qend qlen sseqid sstart send length nident sstrand',
         '-out', str(output_path)
@@ -53,13 +60,13 @@ def predict_oris(genome, contigs_path, ori_type):
                 'contig': cols[4],
                 'contig_start': int(cols[5]),
                 'contig_stop': int(cols[6]),
-                'strand': '+' if cols[9] == 'plus' else '-',
+                'strand': bc.STRAND_FORWARD if cols[9] == 'plus' else bc.STRAND_REVERSE,
                 'coverage': int(cols[7]) / int(cols[3]),
                 'identity': int(cols[8]) / int(cols[7])
             }
-            if(hit['strand'] == '-'):
+            if(hit['strand'] == bc.STRAND_REVERSE):
                 hit['contig_start'], hit['contig_stop'] = hit['contig_stop'], hit['contig_start']
-            if(hit['coverage'] >= 0.8 and hit['identity'] >= 0.8):
+            if(hit['coverage'] >= HIT_COVERAGE and hit['identity'] >= HIT_IDENTITY):
                 contig_hits = hits.get(hit['contig'], [])
                 contig_hits.append(hit)
                 if(len(contig_hits) == 1):
@@ -118,7 +125,7 @@ def predict_oris(genome, contigs_path, ori_type):
     return oris
 
 
-def refine_ori_region(region_hits, ori):
+def refine_ori_region(region_hits: Sequence[int], ori: dict):
     log.debug('refine ori: [%i-%i]', ori['start'], ori['stop'])
     hit_region = region_hits[ori['start']:ori['stop'] + 1]
     hit_min = min(hit_region)
