@@ -1,5 +1,8 @@
 import logging
 
+from pathlib import Path
+from typing import Dict, Sequence, Union
+
 import bakta
 import bakta.config as cfg
 import bakta.constants as bc
@@ -11,7 +14,7 @@ import bakta.so as so
 log = logging.getLogger('GFF')
 
 
-def write_gff3(genome, features_by_contig, gff3_path):
+def write_gff3(genome: dict, features_by_contig: Dict[str, dict], gff3_path: Path):
     """Export features in GFF3 format."""
     log.info('write GFF3: path=%s', gff3_path)
 
@@ -210,6 +213,16 @@ def write_gff3(genome, features_by_contig, gff3_path):
                         annotations['Note'] = [note for note in annotations['Note'] if bc.DB_XREF_EC not in note]
                         gene_annotations = encode_annotations(gene_annotations)
                         fh.write(f"{feat['contig']}\tProdigal\tgene\t{start}\t{stop}\t.\t{feat['strand']}\t.\t{gene_annotations}\n")
+                    if('exception' in feat):
+                        ex = feat['exception']
+                        pos = f"{ex['start']}..{ex['stop']}"
+                        if(feat['strand'] == bc.STRAND_REVERSE):
+                            pos = f"complement({pos})"
+                        annotations['transl_except']=f"(pos:{pos},aa:{ex['aa']})"
+                        notes = annotations.get('Note', [])
+                        notes.append(f"codon on position {ex['codon_position']} is a {ex['type']} codon")
+                        if('Notes' not in annotations):
+                            annotations['Note'] = notes
                     annotations = encode_annotations(annotations)
                     fh.write(f"{feat['contig']}\tProdigal\t{so.SO_CDS.name}\t{start}\t{stop}\t.\t{feat['strand']}\t0\t{annotations}\n")
                     if(bc.FEATURE_SIGNAL_PEPTIDE in feat):
@@ -299,7 +312,7 @@ def write_gff3(genome, features_by_contig, gff3_path):
     return
 
 
-def encode_attribute(product):
+def encode_attribute(product: str) -> str:
     """Replace special characters forbidden in column 9 of the GFF3 format: https://github.com/The-Sequence-Ontology/Specifications/blob/master/gff3.md"""
     product = str(product)
     product = product.replace('%', '%25')
@@ -310,7 +323,7 @@ def encode_attribute(product):
     return product
 
 
-def encode_annotations(annotations):
+def encode_annotations(annotations: Dict[str, Union[str, Sequence[str]]]) -> str:
     annotation_strings = []
     for key, val in annotations.items():
         if(type(val) is list):
@@ -323,7 +336,7 @@ def encode_annotations(annotations):
     return ';'.join(annotation_strings)
 
 
-def write_signal_peptide(fh, feat):
+def write_signal_peptide(fh, feat: dict):
     sig_peptide = feat[bc.FEATURE_SIGNAL_PEPTIDE]
     annotations = {
         'ID': f"{feat['locus']}_sigpep",
