@@ -2,6 +2,8 @@ import logging
 import re
 
 from datetime import date, datetime
+from pathlib import Path
+from typing import Sequence, Tuple
 
 from Bio import SeqIO
 from Bio.Seq import Seq
@@ -18,7 +20,7 @@ import bakta.so as so
 log = logging.getLogger('INSDC')
 
 
-def write_insdc(genome, features, genbank_output_path, embl_output_path):
+def write_insdc(genome: dict, features:Sequence[dict], genbank_output_path: Path, embl_output_path: Path):
     log.debug('prepare: genbank=%s, embl=%s', genbank_output_path, embl_output_path)
 
     contig_list = []
@@ -91,7 +93,9 @@ def write_insdc(genome, features, genbank_output_path, embl_output_path):
 
         for feature in contig_features:
             insdc_feature_type = None
-            qualifiers = {}
+            qualifiers = {
+                'note': []
+            }
             if('db_xrefs' in feature):
                 if(cfg.compliant):
                     qualifiers['db_xref'], qualifiers['note'] = revise_dbxref_insdc(feature['db_xrefs'])
@@ -150,6 +154,13 @@ def write_insdc(genome, features, genbank_output_path, embl_output_path):
                         if(bc.DB_XREF_EC in note):
                             qualifiers['EC_number'] = note.replace('EC:', '')
                     qualifiers['note'] = [note for note in qualifiers['note'] if bc.DB_XREF_EC not in note]
+                if('exception' in feature):
+                    ex = feature['exception']
+                    pos = f"{ex['start']}..{ex['stop']}"
+                    if(feature['strand'] == bc.STRAND_REVERSE):
+                        pos = f"complement({pos})"
+                    qualifiers['transl_except']=f"(pos:{pos},aa:{ex['aa']})"
+                    qualifiers['note'].append(f"codon on position {ex['codon_position']} is a {ex['type']} codon")
                 if(bc.FEATURE_SIGNAL_PEPTIDE in feature):
                     sigpep_qualifiers = {}
                     sigpep_qualifiers['locus_tag'] = feature['locus']
@@ -263,7 +274,7 @@ def write_insdc(genome, features, genbank_output_path, embl_output_path):
         SeqIO.write(contig_list, fh, format='embl')
 
 
-def select_ncrna_class(feature):
+def select_ncrna_class(feature: dict) -> str:
     if(feature['class'] is None):
         return bc.INSDC_FEATURE_NC_RNA_CLASS_OTHER
     elif(feature['class'].id == so.SO_NCRNA_GENE_ANTISENSE.id):
@@ -276,7 +287,7 @@ def select_ncrna_class(feature):
         return bc.INSDC_FEATURE_NC_RNA_CLASS_OTHER
 
 
-def select_regulatory_class(feature):
+def select_regulatory_class(feature: dict) -> str:
     if(feature['class'] is None):
         return bc.INSDC_FEATURE_REGULATORY_CLASS_OTHER
     elif(feature['class'].id == so.SO_CIS_REG_ATTENUATOR.id):
@@ -293,7 +304,7 @@ def select_regulatory_class(feature):
         return bc.INSDC_FEATURE_REGULATORY_CLASS_OTHER
 
 
-def revise_product_insdc(feature):
+def revise_product_insdc(feature: dict):
     """Revise product name for INSDC compliant submissions"""
     product = feature['product']
 
@@ -315,7 +326,7 @@ def revise_product_insdc(feature):
     feature['product'] = product
 
 
-def revise_dbxref_insdc(dbxrefs):
+def revise_dbxref_insdc(dbxrefs: Sequence[str]) -> Tuple[Sequence[str], Sequence[str]]:
     """Remove INSDC non-compliant DbXrefs."""
     insdc_valid_dbxrefs = [bc.DB_XREF_UNIPROTKB, bc.DB_XREF_GO, bc.DB_XREF_PFAM, bc.DB_XREF_RFAM]
     valid_dbxrefs = []
