@@ -3,7 +3,6 @@ import argparse
 import logging
 from math import remainder
 import os
-import shutil
 import sys
 import platform as pf
 
@@ -48,33 +47,9 @@ def main():
     # Setup logging
     ############################################################################
     cfg.prefix = args.prefix if args.prefix else Path(args.genome).stem
-    try:
-        output_path = Path(args.output) if args.output else Path.cwd()
-        if(not output_path.exists()):
-            output_path.mkdir(parents=True, exist_ok=True)
-        elif(not os.access(str(output_path), os.X_OK)):
-            sys.exit(f'ERROR: output path ({output_path}) not accessible!')
-        elif(not os.access(str(output_path), os.W_OK)):
-            sys.exit(f'ERROR: output path ({output_path}) not writable!')
-        output_path = output_path.resolve()
-        cfg.output_path = output_path
-    except:
-        sys.exit(f'ERROR: could not resolve or create output directory ({args.output})!')
-    logging.basicConfig(
-        filename=str(output_path.joinpath(f'{cfg.prefix}.log')),
-        filemode='w',
-        format='%(asctime)s.%(msecs)03d - %(levelname)s - %(name)s - %(message)s',
-        datefmt='%H:%M:%S',
-        level=logging.DEBUG if args.verbose else logging.INFO
-    )
+    output_path = cfg.check_output_path(args)
+    bu.setup_logger(output_path, cfg.prefix, args)
     log = logging.getLogger('MAIN')
-    log.info('version=%s', bakta.__version__)
-    log.info('developer: Oliver Schwengers, github.com/oschwengers')
-    log.info('command: %s', ' '.join(sys.argv))
-    log.info('local time: %s', datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
-    log.info('machine: type=%s, cores=%s', pf.processor(), os.cpu_count())
-    log.info('system: type=%s, release=%s', pf.system(), pf.release())
-    log.info('python: version=%s, implementation=%s', pf.python_version(), pf.python_implementation())
 
     ############################################################################
     # Checks and configurations
@@ -83,7 +58,7 @@ def main():
     # - test binary dependencies
     ############################################################################
     cfg.setup(args)  # check parameters and prepare global configuration
-    atexit.register(cleanup, log, cfg.tmp_path)  # register cleanup exit hook
+    atexit.register(bu.cleanup, log, cfg.tmp_path)  # register cleanup exit hook
     cfg.db_info = db.check(cfg.db_path)
     bu.test_dependencies()
     if(cfg.verbose):
@@ -254,6 +229,12 @@ def main():
             discarded_cdss = orf.detect_spurious(cdss, cds_aa_path)
             print(f'\tdiscarded spurious: {len(discarded_cdss)}')
             cdss = [cds for cds in cdss if 'discarded' not in cds]
+        
+        if(len(cdss) > 0):
+            log.debug('revise translational exceptions')
+            no_revised = feat_cds.revise_translational_exceptions(genome, cdss)
+            print(f'\trevised translational exceptions: {no_revised}')
+            cdss = [cds for cds in cdss if 'discarded' not in cds]
 
         if(len(cdss) > 0):
             log.debug('lookup CDS UPS/IPS')
@@ -309,7 +290,7 @@ def main():
                 print('\tcalculated proteins statistics')
             
             print('\trevise special cases...')
-            feat_cds.revise_special_cases(cdss)
+            feat_cds.revise_special_cases_annotated(genome, cdss)
 
         genome['features'][bc.FEATURE_CDS] = cdss
 
@@ -537,7 +518,7 @@ def main():
         hypotheticals = [feat for feat in features if feat['type'] == bc.FEATURE_CDS and 'hypothetical' in feat]
         print('\thypothetical TSV...')
         tsv_path = cfg.output_path.joinpath(f'{cfg.prefix}.hypotheticals.tsv')
-        tsv.write_hypothetical_tsv(hypotheticals, tsv_path)
+        tsv.write_hypotheticals_tsv(hypotheticals, tsv_path)
 
         print('\ttranslated hypothetical CDS sequences...')
         faa_path = cfg.output_path.joinpath(f'{cfg.prefix}.hypotheticals.faa')
@@ -581,10 +562,6 @@ def main():
     print(f'\nIf you use these results please cite Bakta: https://doi.org/{bc.BAKTA_DOI}')
     print(f'Annotation successfully finished in {int(run_duration / 60):01}:{int(run_duration % 60):02} [mm:ss].')
 
-def cleanup(log, tmp_path):
-    shutil.rmtree(str(tmp_path))  # remove tmp dir
-    log.info('removed tmp dir: %s', tmp_path)
-
 
 if __name__ == '__main__':
     main()
