@@ -5,10 +5,17 @@ import hashlib
 import logging
 import multiprocessing as mp
 import os
+import shutil
+import platform as pf
 import sys
 import subprocess as sp
 import re
 
+from argparse import Namespace
+from datetime import datetime
+from typing import Dict, Sequence, Tuple
+from pathlib import Path
+
 from Bio.Seq import Seq
 
 import bakta
@@ -32,21 +39,21 @@ VERSION_MAX_DIGIT = 1000000000000
 VERSION_REGEX = re.compile(r'(\d+)\.(\d+)(?:\.(\d+))?')  # regex to search for version number in tool output. Takes missing patch version into consideration.
 
 # List of dependencies: tuples for: min version, max version, tool name & command line parameter, dependency check exclusion options
-DEPENDENCY_TRNASCAN = (Version(2, 0, 6), Version(VERSION_MAX_DIGIT, VERSION_MAX_DIGIT, VERSION_MAX_DIGIT), VERSION_REGEX, ('tRNAscan-SE', '-h'), ['--skip-trna'])
+DEPENDENCY_TRNASCAN = (Version(2, 0, 8), Version(VERSION_MAX_DIGIT, VERSION_MAX_DIGIT, VERSION_MAX_DIGIT), VERSION_REGEX, ('tRNAscan-SE', '-h'), ['--skip-trna'])
 DEPENDENCY_ARAGORN = (Version(1, 2, 38), Version(VERSION_MAX_DIGIT, VERSION_MAX_DIGIT, VERSION_MAX_DIGIT), VERSION_REGEX, ('aragorn', '-h'), ['skip-tmrna'])
 DEPENDENCY_CMSCAN = (Version(1, 1, 4), Version(VERSION_MAX_DIGIT, VERSION_MAX_DIGIT, VERSION_MAX_DIGIT), VERSION_REGEX, ('cmscan', '-h'), ['--skip-rrna', '--skip-ncrna', '--skip-ncrna-region'])
 DEPENDENCY_PILERCR = (Version(1, 6), Version(VERSION_MAX_DIGIT, VERSION_MAX_DIGIT, VERSION_MAX_DIGIT), VERSION_REGEX, ('pilercr', '-options'), ['--skip-crispr'])
 DEPENDENCY_PRODIGAL = (Version(2, 6, 3), Version(VERSION_MAX_DIGIT, VERSION_MAX_DIGIT, VERSION_MAX_DIGIT), VERSION_REGEX, ('prodigal', '-v'), ['--skip-cds'])
-DEPENDENCY_HMMSEARCH = (Version(3, 3, 1), Version(VERSION_MAX_DIGIT, VERSION_MAX_DIGIT, VERSION_MAX_DIGIT), VERSION_REGEX, ('hmmsearch', '-h'), ['--skip-cds', '--skip-sorf'])
-DEPENDENCY_DIAMOND = (Version(2, 0, 11), Version(VERSION_MAX_DIGIT, VERSION_MAX_DIGIT, VERSION_MAX_DIGIT), VERSION_REGEX, ('diamond', 'help'), ['--skip-cds', '--skip-sorf'])
+DEPENDENCY_HMMSEARCH = (Version(3, 3, 2), Version(VERSION_MAX_DIGIT, VERSION_MAX_DIGIT, VERSION_MAX_DIGIT), VERSION_REGEX, ('hmmsearch', '-h'), ['--skip-cds', '--skip-sorf'])
+DEPENDENCY_DIAMOND = (Version(2, 0, 14), Version(VERSION_MAX_DIGIT, VERSION_MAX_DIGIT, VERSION_MAX_DIGIT), VERSION_REGEX, ('diamond', 'help'), ['--skip-cds', '--skip-sorf'])
 DEPENDENCY_DEEPSIG = (Version(1, 2, 5), Version(VERSION_MAX_DIGIT, VERSION_MAX_DIGIT, VERSION_MAX_DIGIT), VERSION_REGEX, ('deepsig', '--version'), ['--gram ?'])
-DEPENDENCY_BLASTN = (Version(2, 7, 1), Version(VERSION_MAX_DIGIT, VERSION_MAX_DIGIT, VERSION_MAX_DIGIT), VERSION_REGEX, ('blastn', '-version'), ['--skip-ori'])
-DEPENDENCY_AMRFINDERPLUS = (Version(3, 10, 16), Version(VERSION_MAX_DIGIT, VERSION_MAX_DIGIT, VERSION_MAX_DIGIT), VERSION_REGEX, ('amrfinder', '--version'), ['--skip-cds'])
+DEPENDENCY_BLASTN = (Version(2, 12, 0), Version(VERSION_MAX_DIGIT, VERSION_MAX_DIGIT, VERSION_MAX_DIGIT), VERSION_REGEX, ('blastn', '-version'), ['--skip-ori'])
+DEPENDENCY_AMRFINDERPLUS = (Version(3, 10, 23), Version(VERSION_MAX_DIGIT, VERSION_MAX_DIGIT, VERSION_MAX_DIGIT), VERSION_REGEX, ('amrfinder', '--version'), ['--skip-cds'])
 
 
-def init_parser():
+def init_parser(sub_command: str=''):
     parser = argparse.ArgumentParser(
-        prog='bakta',
+        prog=f'bakta{sub_command}',
         description='Rapid & standardized annotation of bacterial genomes, MAGs & plasmids',
         epilog=f'Version: {bakta.__version__}\nDOI: {bc.BAKTA_DOI}\nURL: github.com/oschwengers/bakta\n\nCitation:\n{bc.BAKTA_CITATION}',
         formatter_class=argparse.RawDescriptionHelpFormatter,
@@ -104,6 +111,28 @@ def parse_arguments():
     return parser.parse_args()
 
 
+def setup_logger(output_path: Path, prefix: str, args: Namespace):
+    logging.basicConfig(
+        filename=str(output_path.joinpath(f'{prefix}.log')),
+        filemode='w',
+        format='%(asctime)s.%(msecs)03d - %(levelname)s - %(name)s - %(message)s',
+        datefmt='%H:%M:%S',
+        level=logging.DEBUG if args.verbose else logging.INFO
+    )
+    log.info('version=%s', bakta.__version__)
+    log.info('developer: Oliver Schwengers, github.com/oschwengers')
+    log.info('command: %s', ' '.join(sys.argv))
+    log.info('local time: %s', datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
+    log.info('machine: type=%s, cores=%s', pf.processor(), os.cpu_count())
+    log.info('system: type=%s, release=%s', pf.system(), pf.release())
+    log.info('python: version=%s, implementation=%s', pf.python_version(), pf.python_implementation())
+
+
+def cleanup(log, tmp_path: Path):
+    shutil.rmtree(str(tmp_path))  # remove tmp dir
+    log.info('removed tmp dir: %s', tmp_path)
+
+
 def read_tool_output(dependency):
     """Method for reading tool version with regex. Input: regex expression, tool command. Retursn: version number."""
     version_regex = dependency[2]
@@ -140,7 +169,7 @@ def read_tool_output(dependency):
         sys.exit('ERROR: Could not detect/read %s version!', command[0])
 
 
-def check_version(tool, min, max):
+def check_version(tool, min: int, max:int ) -> bool:
     """Method for checking tool versions with required version. Input: tool version, minimum and maximum version. Returns: boolean value for positive or negative check."""
     if tool.major < min.major or tool.major > max.major:
         return False
@@ -177,19 +206,19 @@ def test_dependency(dependency):
 
 def test_dependencies():
     """Test the proper installation of all required 3rd party executables."""
-    if(not cfg.skip_trna):
+    if(cfg.skip_trna is not None and cfg.skip_trna is False):
         test_dependency(DEPENDENCY_TRNASCAN)
 
-    if(not cfg.skip_tmrna):
+    if(cfg.skip_tmrna is not None and cfg.skip_tmrna is False):
         test_dependency(DEPENDENCY_ARAGORN)
 
-    if(not cfg.skip_rrna or not cfg.skip_ncrna or not cfg.skip_ncrna_region):
+    if((cfg.skip_rrna is not None and cfg.skip_rrna is False) or (cfg.skip_ncrna is not None and cfg.skip_ncrna is False) or (cfg.skip_ncrna_region is not None and cfg.skip_ncrna_region is False)):
         test_dependency(DEPENDENCY_CMSCAN)
 
-    if(not cfg.skip_crispr):
+    if(cfg.skip_crispr is not None and cfg.skip_crispr is False):
         test_dependency(DEPENDENCY_PILERCR)
 
-    if(not cfg.skip_cds):
+    if(cfg.skip_cds is not None and cfg.skip_cds is False):
         test_dependency(DEPENDENCY_PRODIGAL)
         test_dependency(DEPENDENCY_AMRFINDERPLUS)
 
@@ -206,17 +235,17 @@ def test_dependencies():
             log.error('AMRFinderPlus database not installed')
             sys.exit("ERROR: AMRFinderPlus database not installed! Please, install AMRFinderPlus's internal database by executing: 'amrfinder_update --database <database-path>/amrfinderplus-db'. This must be done only once.")
 
-    if(not cfg.skip_cds or not cfg.skip_sorf):
+    if((cfg.skip_cds is not None and cfg.skip_cds is False) or (cfg.skip_sorf is not None and cfg.skip_sorf is False)):
         test_dependency(DEPENDENCY_HMMSEARCH)
         test_dependency(DEPENDENCY_DIAMOND)
-        if(cfg.gram != '?'):
+        if(cfg.gram is not None and cfg.gram != '?'):
             test_dependency(DEPENDENCY_DEEPSIG)
 
-    if(not cfg.skip_ori):
+    if(cfg.skip_ori is not None and cfg.skip_ori is False):
         test_dependency(DEPENDENCY_BLASTN)
 
 
-def create_locus_tag_prefix(contigs, length=6):
+def create_locus_tag_prefix(contigs: Sequence[dict], length: int=6) -> str:
     """Create either genus/species or sequence MD5 hex based locus tag prefix."""
     hash = hashlib.md5()
     for contig in contigs:
@@ -235,12 +264,12 @@ def create_locus_tag_prefix(contigs, length=6):
     return locus_prefix
 
 
-def calc_aa_hash(seq):
+def calc_aa_hash(seq: str) -> Tuple[bytes, str]:
     aa_hash = hashlib.md5(seq.encode('utf-8'))
     return (aa_hash.digest(), aa_hash.hexdigest())
 
 
-def has_annotation(feature, attribute):
+def has_annotation(feature: dict, attribute: str) -> bool:
     value = feature.get(attribute, None)
     if(value is not None and value != ''):
         return True
@@ -248,7 +277,7 @@ def has_annotation(feature, attribute):
         return False
 
 
-def calc_genome_stats(genome, features):
+def calc_genome_stats(genome: dict, features: Sequence[dict]):
     genome_size = genome['size']
     log.info('genome-size=%i', genome_size)
 
@@ -298,7 +327,7 @@ def calc_genome_stats(genome, features):
     }
 
 
-def parse_replicon_table(replicon_table_path):
+def parse_replicon_table(replicon_table_path: Path) -> Dict[str, dict]:
     replicons = {}
     try:
         with replicon_table_path.open() as fh:
@@ -344,7 +373,7 @@ def parse_replicon_table(replicon_table_path):
     return replicons
 
 
-def qc_contigs(contigs, replicons):
+def qc_contigs(contigs: Sequence[dict], replicons: Dict[str, dict]) -> Tuple[Sequence[dict], bool]:
     valid_contigs = []
     contig_counter = 1
     contig_prefix = cfg.locus if cfg.locus else 'contig'
@@ -452,7 +481,7 @@ def qc_contigs(contigs, replicons):
     return valid_contigs, complete_genome
 
 
-def extract_feature_sequence(feature, contig):
+def extract_feature_sequence(feature: dict, contig: dict) -> str:
     if(feature.get('edge', False)):
         seq = contig['sequence'][feature['start']-1:] + contig['sequence'][:feature['stop']]
     else:
